# Standard library imports.
import datetime

# Pytz package imports.
import pytz

# Django package imports.
from django import forms
from django.utils.timezone import now


# These are the frequency ID values used to retrieve data from the 
# django-epdb database and then to display the data through this app.
FIVEMIN_FREQUENCY_ID = 'fivemin'
HOURLY_FREQUENCY_ID = 'hourly'
DAILY_FREQUENCY_ID = 'daily'
FREQUENCY_CHOICES = ((DAILY_FREQUENCY_ID, 'Daily'), 
    (HOURLY_FREQUENCY_ID, 'Hourly'),
    (FIVEMIN_FREQUENCY_ID, 'Five Minute'),)

# Data output constants.  These constants are used in the request and 
# validation code for viewing data.
GRAPHS_OUTPUT = 'graphs'
TABLE_OUTPUT = 'table'
CSV_OUTPUT = 'csv'
OUTPUT_CHOICES = ((GRAPHS_OUTPUT, 'Graphs'), 
    (TABLE_OUTPUT, 'HTML Table'),
    (CSV_OUTPUT, 'CSV Data'),)

# Data units constants.  These constants are used in the request code 
# for viewing data and provide the ability to choose whether data will 
# be viewed in imperial or metric units.
IMPERIAL_UNITS = 'imperial'
METRIC_UNITS = 'metric'
UNITS_CHOICES = ((IMPERIAL_UNITS, 'English'), 
    (METRIC_UNITS, 'Metric'))


class StationBase(forms.Form):

    station_id = forms.CharField()
    frequency_id = forms.ChoiceField(choices = FREQUENCY_CHOICES, 
        initial = DAILY_FREQUENCY_ID)
    time_zone = forms.CharField()
    units = forms.ChoiceField(choices = UNITS_CHOICES, 
        initial = IMPERIAL_UNITS)
    output = forms.ChoiceField(choices = OUTPUT_CHOICES, 
        initial = GRAPHS_OUTPUT)

    def clean(self):

        cleaned_data = super(StationBase, self).clean()
        time_zone = cleaned_data.get('time_zone')

        # If the 'time_zone' contains slashes, then make it URL safe 
        # by replacing those slashes.
        url_safe_time_zone = time_zone.replace('/', '--SLASH--')
        cleaned_data['url_safe_time_zone'] = url_safe_time_zone

        # Convert the cleaned URL safe 'time_zone' field into an actual 
        # Olson time zone.  If the time zone does not exits, a form 
        # 'ValidationError' is raised.  Replace original 'time_zone' 
        # entry with the actual Olson time zone in the 'cleaned_data' 
        # dictionary.
        time_zone = time_zone.replace('--SLASH--', '/')

        if time_zone not in pytz.all_timezones:

            msg = 'The given time zone does not exist.'
            raise forms.ValidationError(msg)

        cleaned_data['time_zone'] = time_zone

        if cleaned_data.get('units') == IMPERIAL_UNITS:
            cleaned_data['imperial'] = True

        else:
            cleaned_data['imperial'] = False

        return cleaned_data


class StationTimeseries(StationBase):

    start_date = forms.DateField(input_formats = ['%Y-%m-%d'])
    end_date = forms.DateField(input_formats = ['%Y-%m-%d'])

    def clean(self):

        cleaned_data = super(StationTimeseries, self).clean()
        frequency_id = cleaned_data.get('frequency_id')
        time_zone = cleaned_data.get('time_zone')
        units = cleaned_data.get('units')
        output = cleaned_data.get('output')
        sd = cleaned_data.get('start_date')
        ed = cleaned_data.get('end_date')

        # Check if the start date is greater than the end date.  Raise 
        # a 'ValidationError' if True.
        if sd > ed:

            msg = 'Start date is greater than the end date.'
            raise forms.ValidationError(msg)

        # The difference between the end date and start date.  The 
        # value should be positive.
        delta = ed - sd

        if output == GRAPHS_OUTPUT:

            if delta > datetime.timedelta(days = 7) and \
                frequency_id == FIVEMIN_FREQUENCY_ID:

                msg = 'Graph output for five minute data is ' + \
                    'not allowed for date ranges greater than ' + \
                    'one week!'
                raise forms.ValidationError(msg)

            if delta > datetime.timedelta(days = 31) and \
                frequency_id == HOURLY_FREQUENCY_ID:

                msg = 'Graph output for hourly data is not ' + \
                    'allowed for date ranges greater than ' + \
                    'one month!'
                raise forms.ValidationError(msg)

            if delta > datetime.timedelta(days = 366) and \
                frequency_id == DAILY_FREQUENCY_ID:

                msg = 'Graph output is not allowed for date ' + \
                    'ranges greater than one year!'
                raise forms.ValidationError(msg)

        else:

            if delta > datetime.timedelta(days = 31) and \
                frequency_id == FIVEMIN_FREQUENCY_ID:

                msg = 'HTML and CSV output for five minute ' + \
                    'data is not allowed for date ranges ' + \
                    'greater than one month!'
                raise forms.ValidationError(msg)

            if delta > datetime.timedelta(days = 366) and \
                frequency_id == HOURLY_FREQUENCY_ID:

                msg = 'HTML and CSV output for hourly data ' + \
                    'is not allowed for date ranges greater ' + \
                    'than one year!'
                raise forms.ValidationError(msg)

            if delta > datetime.timedelta(days = 1830) and \
                frequency_id == DAILY_FREQUENCY_ID:

                msg = 'HTML and CSV output is not allowed ' + \
                    'for date ranges greater than five years!'
                raise forms.ValidationError(msg)

        # Determine the start and end datetimes, and save the results 
        # in the cleaned_data dictionary.
        sdt = datetime.datetime(sd.year, sd.month, sd.day)
        edt = datetime.datetime(ed.year, ed.month, ed.day)

        tzinfo = pytz.timezone(time_zone)
        sdt = tzinfo.localize(sdt)
        edt = tzinfo.localize(edt)

        if frequency_id == FIVEMIN_FREQUENCY_ID:

            sdt += datetime.timedelta(seconds = 300)
            edt += datetime.timedelta(days = 1)

        elif frequency_id == HOURLY_FREQUENCY_ID:

            sdt += datetime.timedelta(seconds = 3600)
            edt += datetime.timedelta(days = 1)

        cleaned_data['sdt'] = sdt
        cleaned_data['edt'] = edt

        # Set a boolean flag in the 'cleaned_data' dictionary that 
        # specifies if imperial units were requested or not.
        cleaned_data['is_imperial_units'] = False

        if units == IMPERIAL_UNITS:
            cleaned_data['is_imperial_units'] = True

        return cleaned_data


class StationTimeseriesPrevious(StationBase):

    previous_days = forms.IntegerField(min_value = 1, max_value = 8)

    def clean(self):

        cleaned_data = super(StationTimeseriesPrevious, self).clean()
        frequency_id = cleaned_data.get('frequency_id')
        time_zone = cleaned_data.get('time_zone')
        units = cleaned_data.get('units')
        previous_days = cleaned_data.get('previous_days')

        # This should be failing earlier if the 'previous_days' field 
        # value is above 8.  Why do I have to check for the existence 
        # of 'previous_days' in the 'cleaned_data' dictionary?
        if not previous_days:

            msg = 'Previous days not within allowable range.'
            raise forms.ValidationError(msg)

        # Get the current UTC datetime and convert it to the "local" 
        # time zone provided by the 'time_zone' value.
        tzinfo = pytz.timezone(time_zone)
        utcnow = now()
        tznow = utcnow.astimezone(tzinfo)

        # Create the start and end dates, using the current localized 
        # datetime and the 'previous_days' value.
        ed = tznow.date()
        sd = ed - datetime.timedelta(days = (previous_days - 1))

        # Determine the start and end datetimes, and save the results 
        # in the cleaned_data dictionary.
        sdt = datetime.datetime(sd.year, sd.month, sd.day)
        edt = datetime.datetime(ed.year, ed.month, ed.day)
        sdt = tzinfo.localize(sdt)
        edt = tzinfo.localize(edt)

        # Commenting this out for now, because, as the views are 
        # designed right now, this form is validated, then using the 
        # cleaned data from this form, the StationDataRange form is 
        # instanciated and validated.  Thus, the end date for hourly 
        # and five minute data creeps up an extra day.
        #if frequency_id == FIVEMIN_FREQUENCY_ID:

        #    sdt += datetime.timedelta(seconds = 300)
        #    edt += datetime.timedelta(days = 1)

        #elif frequency_id == HOURLY_FREQUENCY_ID:

        #    sdt += datetime.timedelta(seconds = 3600)
        #    edt += datetime.timedelta(days = 1)

        cleaned_data['sdt'] = sdt
        cleaned_data['edt'] = edt

        cleaned_data['start_date'] = sdt.date()
        cleaned_data['end_date'] = edt.date()

        # Set a boolean flag in the 'cleaned_data' dictionary that 
        # specifies if imperial units were requested or not.
        cleaned_data['is_imperial_units'] = False

        if units == IMPERIAL_UNITS:
            cleaned_data['is_imperial_units'] = True

        return cleaned_data


class StationRGTimeseries(StationTimeseries):

    frequency_id = forms.ChoiceField(
        choices = ((DAILY_FREQUENCY_ID, 'Daily'),), 
        initial = DAILY_FREQUENCY_ID)
    output = forms.ChoiceField(
        choices = ((CSV_OUTPUT, 'CSV'), 
            (TABLE_OUTPUT, 'HTML Table')), 
        initial = TABLE_OUTPUT)
    gdd_tbase = forms.FloatField(initial = 50.0,
        label = 'Base Temperature')
    gdd_tmaxcutoff = forms.FloatField(initial = 999.0, 
        label = 'Max Cutoff Temperature')
    gdd_tmincutoff = forms.FloatField(initial = -999.0, 
        label = 'Min Cutoff Temperature')
    cgdd_offset = forms.FloatField(initial = 0.0, 
        label = 'Cumulative GDD Offset')

    def clean(self):

        cleaned_data = super(StationRGTimeseries, self).clean()

        if cleaned_data['units'] == IMPERIAL_UNITS:

            f = lambda x: (x - 32.0) * (5.0 / 9.0)
            cleaned_data['gdd_tbase'] = f(cleaned_data['gdd_tbase'])

            if cleaned_data['gdd_tmaxcutoff'] != 999.0:
                cleaned_data['gdd_tmaxcutoff'] = \
                    f(cleaned_data['gdd_tmaxcutoff'])

            if cleaned_data['gdd_tmincutoff'] != -999.0:
                cleaned_data['gdd_tmincutoff'] = \
                    f(cleaned_data['gdd_tmincutoff'])

        return cleaned_data


class StationRGTimeseriesPrevious(StationTimeseriesPrevious):

    frequency_id = forms.ChoiceField(
        choices = ((DAILY_FREQUENCY_ID, 'Daily'),), 
        initial = DAILY_FREQUENCY_ID)
    output = forms.ChoiceField(
        choices = ((CSV_OUTPUT, 'CSV'), 
            (TABLE_OUTPUT, 'HTML Table')), 
        initial = TABLE_OUTPUT)
    gdd_tbase = forms.FloatField(initial = 50.0,
        label = 'Base Temperature')
    gdd_tmaxcutoff = forms.FloatField(initial = 999.0, 
        label = 'Max Cutoff Temperature')
    gdd_tmincutoff = forms.FloatField(initial = -999.0, 
        label = 'Min Cutoff Temperature')
    cgdd_offset = forms.FloatField(initial = 0.0, 
        label = 'Cumulative GDD Offset')

    def clean(self):

        cleaned_data = super(StationRGTimeseriesPrevious, self).clean()

        if cleaned_data['units'] == IMPERIAL_UNITS:

            f = lambda x: (x - 32.0) * (5.0 / 9.0)
            cleaned_data['gdd_tbase'] = f(cleaned_data['gdd_tbase'])

            if cleaned_data['gdd_tmaxcutoff'] != 999.0:
                cleaned_data['gdd_tmaxcutoff'] = \
                    f(cleaned_data['gdd_tmaxcutoff'])

            if cleaned_data['gdd_tmincutoff'] != -999.0:
                cleaned_data['gdd_tmincutoff'] = \
                    f(cleaned_data['gdd_tmincutoff'])

        return cleaned_data


