# Standard library imports.
import datetime
import uuid

# PyTZ package imports.
import pytz

# Django package imports.
from django.db import models


# These are the default start and end datetime for the Station 
# classes.
UTC = pytz.timezone('UTC')
DEFAULT_SDT = datetime.datetime(1900, 1, 1, 0, 0, 0, tzinfo = UTC)
DEFAULT_EDT = datetime.datetime(9999, 12, 31, 23, 59, 59, tzinfo = UTC)


def now():
    return datetime.datetime.utcnow().replace(tzinfo = UTC)


# #####################################################################
# Application model implementations begin here.
# #####################################################################

class BaseModel(models.Model):
    """The abstract base class for the ZiaMet models.

    Attributes
    ----------
    _id : uuid
        This is the primary key field used for database storage.  
    created : datetime
        A UTC datetime that denotes when the model information was 
        stored in the database.
    last_modified : datetime
        A UTC datetime that denotes when the model information was 
        updated in the database.
    description : string
        Stores a description of the model information.  Information 
        stored in this attribute could be anything about the model 
        that is not easily quantifiable.
    """

    _id = models.UUIDField(primary_key = True, default = uuid.uuid4, 
        editable = False)
    created = models.DateTimeField(auto_now_add = True)
    last_modified = models.DateTimeField(auto_now = True)
    description = models.TextField(blank = True, null = True)

    class Meta:
        abstract = True


class Station(BaseModel):
    """Represents a surface weather station.

    This model is used to describe a surface weather station that 
    observes a standard set of variables.

    Attributes
    ----------
    station_id : string
        The public identifier for the Station.
    sdt : datetime
        A UTC datetime that denotes when the Station was deployed.
    edt : datetime
        A UTC datetime that denotes when the Station was discontinued. 
        The default value is the last second of the year 9999.
    name : string
        The name of the Station.
    latitude : float
        The latitude location of the Station, as a decimal number.
    longitude : float
        The longitude location of the Station, as a decimal number.
    elevation : float
        The elevation of the Station, in meters, as a decimal number.
    time_zone : string
        Represents the time zone where the Station is located.

    """
    station_id = models.CharField(max_length = 32, verbose_name = 'Station ID')
    sdt = models.DateTimeField(default = now)
    edt = models.DateTimeField(default = DEFAULT_EDT)
    name = models.CharField(max_length = 256)
    latitude = models.FloatField(default = 32.282)
    longitude = models.FloatField(default = -106.760)
    elevation = models.FloatField(default = 1186.0)
    time_zone = models.CharField(max_length = 64, verbose_name = 'Time Zone', 
        blank = False, null = False, default = 'US/Mountain')

    class Meta:
        unique_together = ('station_id', 'sdt')

    def valid_timestamp(self, timestamp):
        """Determines if a 'timestamp' is valid for the Station.

        The 'timestamp' is checked agains the 'sdt' and 'edt' 
        attributes to determine if the 'timestamp' parameter falls 
        within the datetime range of the 'sdt' and 'edt' attribute 
        values.

        Parameters
        ----------
        timestamp : datetime
            A UTC datetime.

        Returns
        -------
        boolean
            True, if the given 'timestamp' parameter falls within the 
            'sdt' and 'edt' attribute values, False otherwise.
        """
        is_valid = False

        try:
            if (timestamp >= self.sdt) and (timestamp < self.edt):
                is_valid = True

        except:
            pass

        return is_valid


class FiveMinData(BaseModel):

    station = models.ForeignKey('Station')
    timestamp = models.DateTimeField(blank = False, null = False)
    record = models.PositiveIntegerField(blank = True, null = True)
    batt_volts = models.FloatField(blank = True, null = True)
    panel_temp = models.FloatField(blank = True, null = True)
    air_temp = models.FloatField(blank = True, null = True)
    rh = models.FloatField(blank = True, null = True)
    wind_speed = models.FloatField(blank = True, null = True)
    wind_speed_max = models.FloatField(blank = True, null = True)
    wind_speed_mean = models.FloatField(blank = True, null = True)
    wind_dir = models.FloatField(blank = True, null = True)
    wind_dir_mean = models.FloatField(blank = True, null = True)
    precip_total = models.FloatField(blank = True, null = True)
    solar_rad = models.FloatField(blank = True, null = True)
    solar_rad_total = models.FloatField(blank = True, null = True)
    soil_temp = models.FloatField(blank = True, null = True)

    class Meta:
        unique_together = ('station', 'timestamp',)


class HourlyData(BaseModel):

    station = models.ForeignKey('Station')
    timestamp = models.DateTimeField(blank = False, null = False)
    record = models.PositiveIntegerField(blank = True, null = True)
    batt_volts_max = models.FloatField(blank = True, null = True)
    batt_volts_min = models.FloatField(blank = True, null = True)
    batt_volts_mean = models.FloatField(blank = True, null = True)
    panel_temp_max = models.FloatField(blank = True, null = True)
    panel_temp_min = models.FloatField(blank = True, null = True)
    panel_temp_mean = models.FloatField(blank = True, null = True)
    air_temp_max = models.FloatField(blank = True, null = True)
    air_temp_min = models.FloatField(blank = True, null = True)
    air_temp_mean = models.FloatField(blank = True, null = True)
    rh_max = models.FloatField(blank = True, null = True)
    rh_min = models.FloatField(blank = True, null = True)
    rh_mean = models.FloatField(blank = True, null = True)
    wind_speed_max = models.FloatField(blank = True, null = True)
    wind_speed_mean = models.FloatField(blank = True, null = True)
    wind_dir_mean = models.FloatField(blank = True, null = True)
    precip_total = models.FloatField(blank = True, null = True)
    solar_rad_max = models.FloatField(blank = True, null = True)
    solar_rad_min = models.FloatField(blank = True, null = True)
    solar_rad_mean = models.FloatField(blank = True, null = True)
    solar_rad_total = models.FloatField(blank = True, null = True)
    soil_temp_max = models.FloatField(blank = True, null = True)
    soil_temp_min = models.FloatField(blank = True, null = True)
    soil_temp_mean = models.FloatField(blank = True, null = True)

    class Meta:
        unique_together = ('station', 'timestamp',)



