# Standard library imports.
import datetime

# PyTZ package imports.
import pytz

# Calculator module imports.
import dewpoint
import wind_chill
import heat_index
import gdd
import daily_etref

# Numpy package imports.
import numpy as np

# Pandas package imports.
import pandas as pd

# Django package imports.
from django.db.models import Max
from django.db.models.fields import PositiveIntegerField
from django.db.models.fields import FloatField
from django.http import HttpResponse
from django.http import HttpResponseBadRequest
from django.template import loader
from django.template import RequestContext
from django.views.generic import View

# Local package imports.
from .forms import FIVEMIN_FREQUENCY_ID
from .forms import HOURLY_FREQUENCY_ID
from .forms import DAILY_FREQUENCY_ID
from .forms import GRAPHS_OUTPUT
from .forms import TABLE_OUTPUT
from .forms import CSV_OUTPUT
from .forms import IMPERIAL_UNITS
from .forms import StationTimeseries as StationTimeseriesF
from .forms import StationTimeseriesPrevious as StationTimeseriesPreviousF
from .forms import StationRGTimeseries as StationRGTimeseriesF
from .forms import StationRGTimeseriesPrevious as StationRGTimeseriesPreviousF
from .models import UTC
from .models import now
from .models import Station as Stn
from .models import FiveMinData
from .models import HourlyData


class Index(View):

    def get(self, request, *args, **kwargs):

        # Get the ZiaMet stations with the max timestamp from the 
        # hourly data.
        stations = Stn.objects.filter(edt__gte = now())\
            .order_by('name')\
            .annotate(max_timestamp = Max('hourlydata__timestamp'))

        context = {}
        context['stations'] = stations

        tp = 'nmcc_ziamet/index.html'
        t = loader.get_template(tp)
        c = RequestContext(request, context)

        return HttpResponse(t.render(c))


class Status(View):

    def get(self, request, *args, **kwargs):

        # Get the ZiaMet stations with the max timestamp from the 
        # five minute data.
        stations = []
        stations_qs = Stn.objects.filter(edt__gte = now())\
            .order_by('name')\
            .annotate(max_timestamp = Max('fivemindata__timestamp'))

        for station in stations_qs:

            station.status_severity_index = 0
            station.status_issues_list = []
            stations.append(station)

        # Sort the stations according to the severity index.
        stations.sort(key = lambda x: x.status_severity_index)
        stations.reverse()

        context = {}
        context['stations'] = stations

        tp = 'nmcc_ziamet/status.html'
        t = loader.get_template(tp)
        c = RequestContext(request, context)

        return HttpResponse(t.render(c))


class Station(View):

    def __localize_ts(self, st, et, time_zone):

        # Represents the local time zone of the station.
        ltz = pytz.timezone(time_zone)

        # Check if the 'st' and 'et' parameters are naive datetimes 
        # and make them aware if so.
        if (st.tzinfo == None):
            st = st.replace(tzinfo = ltz)

        else:
            st = st.astimezone(ltz)

        if et.tzinfo == None:
            et = et.replace(tzinfo = ltz)

        else:
            et = et.astimezone(ltz)

        return (st, et, ltz)

    def __get_columns(self, model):

        columns = []

        pf_type = type(PositiveIntegerField())
        ff_type = type(FloatField())

        for field in model._meta.get_fields():
            if (type(field) == pf_type) or (type(field) == ff_type):
                columns.append(field.name)

        return columns

    def __get_df(self, prng, columns, data, time_zone, add_loc = False):

        df = pd.DataFrame(index = prng, columns = columns, dtype = float)

        for d in data:

            l_timestamp = d.timestamp.astimezone(pytz.timezone(time_zone))

            try:
                row = df.loc[l_timestamp.strftime('%Y-%m-%d %H:%M')]

            except:
                row = None

            if row is not None:

                for c in columns:

                    if c not in ('latitude', 'longitude', 'elevation',):
                        row[c] = getattr(d, c, float('nan'))

                    else:
                        if add_loc is True:
                            row[c] = getattr(d.station, c, float('nan'))

        return df

    def __to_imperial(self, df):

        for col in df.columns:

            if col.startswith('air_temp') or \
                col.startswith('soil_temp') or \
                col.startswith('dewpoint') or \
                col.startswith('wind_chill') or \
                col.startswith('heat_index'):

                df[col] = (df[col] * (9.0 / 5.0)) + 32.0

            if col.startswith('wind_speed'):
                df[col] = df[col] * 2.237

            if col == 'precip_total':
                df[col] = df[col] / 25.4

            if col == 'gdd':
                df[col] = df[col] * (9.0 / 5.0)

            if col in ('eth', 'eto', 'etr'):
                df[col] = df[col] / 25.4

        return None

    def _df_as_dict(self, df, time_zone, missing_value = float('nan')):

        if type(missing_value) == type(float('nan')):
            for c in df.columns:
                df[c] = np.where(np.isnan(df[c]), missing_value, df[c])

        ltz = pytz.timezone(time_zone)
        tsdict = df.to_dict(orient = 'records')

        count = 0
        num_rows = len(tsdict)

        while count < num_rows:

            idx = df.index[count]
            row = tsdict[count]
            row['timestamp'] = datetime.datetime(idx.year, 
                idx.month, 
                idx.day, 
                idx.hour, 
                idx.minute, 
                idx.second, 
                tzinfo = ltz)

            if type(missing_value) == type(''):
                for c in df.columns:
                    if str(row[c]) == str(float('nan')):
                        row[c] = missing_value

            count +=1 

        return tsdict

    def fivemin_timeseries(self, station_id, st, et, time_zone, 
        imperial = False):

        # The 'df' variable is the return value of the method.  If 
        # there is no data in the database for the given start and 
        # end timestamps, the return value should be None.
        df = None

        # Get localized datetimes and the local tzinfo object.
        (lst, let, ltz) = self.__localize_ts(st, et, time_zone)

        # Get the five minute data using the given 'station_id' and 
        # the UTC 'st' and 'et' datetimes.
        data = FiveMinData.objects\
            .filter(timestamp__gte = lst.astimezone(UTC))\
            .filter(timestamp__lte = let.astimezone(UTC))\
            .filter(station__station_id = station_id)\
            .order_by('timestamp')

        if data.count() > 0:

            ds_lst = (5 - (lst.minute % 5)) * 60
            lst += datetime.timedelta(seconds = ds_lst)
            lst_str = lst.strftime('%Y-%m-%d %H:%M')

            ds_let = (let.minute % 5) * 60
            let -= datetime.timedelta(seconds = ds_let)
            let_str = let.strftime('%Y-%m-%d %H:%M')

            prng = pd.period_range(lst_str, let_str, freq = '5min')
            columns = self.__get_columns(FiveMinData)
            df = self.__get_df(prng, columns, data, time_zone)

            # Calculate the dewpoint from the air temperature and the 
            # relative humidity.
            df['dewpoint'] = df.apply(
                lambda row: dewpoint.calculate(row['air_temp'], row['rh']), 
                axis = 1)

            # Calculate the wind chill from the air temperature and the 
            # wind speed.
            df['wind_chill'] = df.apply(
                lambda row: wind_chill.calculate(
                    row['air_temp'], 
                    row['wind_speed'] * 3.6), 
                axis = 1)

            # Calculate the heat index from the air temperature and the 
            # relative humidity.
            df['heat_index'] = df.apply(
                lambda row: heat_index.calculate(row['air_temp'], row['rh']), 
                axis = 1)

            if imperial == True:
                self.__to_imperial(df)

        return df

    def fivemin_timeseries_as_dict(self, station_id, st, et, time_zone, 
        imperial = False, missing_value = float('nan')):

        tsdict = []

        df = self.fivemin_timeseries(station_id, st, et, time_zone, imperial)

        if df is not None:
            tsdict = self._df_as_dict(df, time_zone, missing_value)

        return tsdict

    def hourly_timeseries(self, station_id, st, et, time_zone, 
        imperial = False):

        # The 'df' variable is the return value of the method.  If 
        # there is no data in the database for the given start and 
        # end timestamps, the return value should be None.
        df = None

        # Get localized datetimes and the local tzinfo object.
        (lst, let, ltz) = self.__localize_ts(st, et, time_zone)

        data = HourlyData.objects\
            .filter(timestamp__gte = lst.astimezone(UTC))\
            .filter(timestamp__lte = let.astimezone(UTC))\
            .filter(station__station_id = station_id)\
            .select_related('station')\
            .order_by('timestamp')

        if data.count() > 0:

            ds_lst = lst.minute * 60
            lst += datetime.timedelta(seconds = ds_lst)
            lst_str = lst.strftime('%Y-%m-%d %H:%M')

            ds_let = let.minute * 60
            let -= datetime.timedelta(seconds = ds_let)
            let_str = let.strftime('%Y-%m-%d %H:%M')

            prng = pd.period_range(lst_str, let_str, freq = 'H')
            columns = self.__get_columns(HourlyData)
            columns.extend(['latitude', 'longitude', 'elevation'])
            df = self.__get_df(prng, columns, data, time_zone, add_loc = True)

            if imperial == True:
                self.__to_imperial(df)

        return df

    def hourly_timeseries_as_dict(self, station_id, st, et, time_zone, 
        imperial = False, missing_value = float('nan')):

        tsdict = []

        df = self.hourly_timeseries(station_id, st, et, time_zone, imperial)

        if df is not None:
            tsdict = self._df_as_dict(df, time_zone, missing_value)

        return tsdict

    def daily_timeseries(self, station_id, st, et, time_zone, 
        imperial = False):

        # The 'df' variable is the return value of the method.  If 
        # there is no data in the database for the given start and 
        # end timestamps, the return value should be None.
        df = None

        # Get localized datetimes and the local tzinfo object.
        (lst, let, ltz) = self.__localize_ts(st, et, time_zone)

        # Set the 'lst' and 'let' variables to the base days.
        lst = datetime.datetime(lst.year, 
            lst.month, 
            lst.day, 
            tzinfo = lst.tzinfo)
        let = datetime.datetime(let.year,
            let.month, 
            let.day, 
            tzinfo = let.tzinfo)

        # Get the hourly data that will then be converted to daily 
        # values.
        adj_lst = (lst + datetime.timedelta(seconds = 3600)).astimezone(UTC)
        adj_let = (let + datetime.timedelta(days = 1)).astimezone(UTC)

        hourly_df = self.hourly_timeseries(station_id, adj_lst, adj_let, 
            time_zone)

        if hourly_df is not None:

            hourly_df.index = hourly_df.index - 1

            r_dict = {}
            r_freq = 'D'

            for col in hourly_df.columns:

                if col.endswith('_max'):
                    r_dict[col] = hourly_df[col].resample(r_freq).max()

                if col.endswith('_min'):
                    r_dict[col] = hourly_df[col].resample(r_freq).min()

                if col.endswith('_mean'):
                    r_dict[col] = hourly_df[col].resample(r_freq).mean()

                if col.endswith('_total'):
                    r_dict[col] = hourly_df[col].resample(r_freq).sum()

                if col in ('latitude', 'longitude', 'elevation',):
                    r_dict[col] = hourly_df[col].resample(r_freq).mean()

            lst_str = lst.strftime('%Y-%m-%d')
            let_str = let.strftime('%Y-%m-%d')
            prng = pd.period_range(lst_str, let_str, freq = r_freq)
            df = pd.DataFrame(r_dict, index = prng)

            if imperial == True:
                self.__to_imperial(df)

        return df

    def daily_timeseries_as_dict(self, station_id, st, et, time_zone, 
        imperial = False, missing_value = float('nan')):

        tsdict = []

        df = self.daily_timeseries(station_id, st, et, time_zone, imperial)

        if df is not None:
            tsdict = self._df_as_dict(df, time_zone, missing_value)

        return tsdict

    def daily_retgdd_timeseries(self, station_id, st, et, time_zone, 
        imperial = False, **kwargs):

        ltz = pytz.timezone(time_zone)
        ts_f = lambda x: datetime.datetime(x.year, 
                x.month, 
                x.day, 
                x.hour, 
                x.minute, 
                x.second, 
                tzinfo = ltz)

        # Define the function that will calculate the reference ET 
        # values row-wise.
        et_f = lambda row: daily_etref\
            .calculate(ts_f(row.name).date(), 
                row['latitude'],
                row['elevation'], 
                row['air_temp_max'], 
                row['air_temp_min'], 
                row['rh_max'], 
                row['rh_min'], 
                row['wind_speed_mean'], 
                row['solar_rad_total'])

        # Define the function that will calculate the growing degree 
        # days row-wise.
        gdd_f = lambda row: gdd.calculate(row['air_temp_max'], 
            row['air_temp_min'], **kwargs)

        # Define the function that will calculate the cumulative 
        # degree days column-wise.
        cgdd_offset = kwargs.get('cgdd_offset', 10.0)
        cgdd_f = lambda x: cgdd_offset + x.cumsum()

        df = self.daily_timeseries(station_id, st, et, time_zone, 
            imperial = False)

        if df is not None:

            # Create the columns for the ETref, then apply the et_f 
            # function row-wise to the dataframe.  I have tried to do 
            # the calculations in one step, but it was not working 
            # properly.
            df['eth'] = df.apply(lambda row: et_f(row)[0], axis = 1)
            df['eto'] = df.apply(lambda row: et_f(row)[1], axis = 1)
            df['etr'] = df.apply(lambda row: et_f(row)[2], axis = 1)

            # Calculate the growing degree days.  Wait until the values 
            # are converted to imperial (if necessary) to calculate the 
            # cumulative gdd.
            df['gdd'] = df.apply(gdd_f, axis = 1)
            df['cgdd'] = float('nan')

            # Convert to imperial if the 'imperial' flag had been set 
            # to true earlier in this function.
            if imperial == True:
                self.__to_imperial(df)

            # Calculate the cumulative sum of the growing degree days, 
            # using a year column to group the data.
            df['year'] = [t.year for t in df.index]
            df['cgdd'] = df.groupby('year')['gdd'].apply(cgdd_f)

        return df

    def daily_retgdd_timeseries_as_dict(self, station_id, st, et, time_zone, 
        imperial = False, missing_value = float('nan'), **kwargs):

        tsdict = []

        df = self.daily_retgdd_timeseries(station_id, st, et, time_zone, 
            imperial, **kwargs)

        if df is not None:
            tsdict = self._df_as_dict(df, time_zone, missing_value)

        return tsdict


class StationIndex(Station):

    def __as_dt(self, period):

        dt = datetime.datetime(period.year, 
            period.month, 
            period.day, 
            period.hour, 
            period.minute)

        return dt

    def __since_midnight(self, station, df):

        ltz = pytz.timezone(station.time_zone)
        lmt = station.max_timestamp.astimezone(ltz)
        lm = datetime.datetime(lmt.year, lmt.month, lmt.day, tzinfo = ltz)
        lm_str = lm.strftime('%Y-%m-%d 00:00')
        lmdf = df[lm_str:]

        sm = {}
        sm['air_temp_max'] = lmdf['air_temp'].max()
        sm['air_temp_max_dt'] = self.__as_dt(lmdf['air_temp'].idxmax())
        sm['air_temp_min'] = lmdf['air_temp'].min()
        sm['air_temp_min_dt'] = self.__as_dt(lmdf['air_temp'].idxmin())
        sm['air_temp_mean'] = lmdf['air_temp'].mean()
        sm['wind_speed_max'] = lmdf['wind_speed_max'].max()
        sm['wind_speed_max_dt'] = self.__as_dt(lmdf['wind_speed_max'].idxmax())
        sm['wind_speed_mean'] = lmdf['wind_speed_mean'].mean()
        sm['wind_dir_mean'] = lmdf['wind_dir_mean'].mean()
        sm['precip_total'] = lmdf['precip_total'].sum()

        return sm

    def get(self, request, *args, **kwargs):

        station_id = kwargs.get('station_id', None)
        station = None

        if station_id is not None:

            stations = Stn.objects.filter(edt__gte = now())\
                .filter(station_id = station_id)\
                .annotate(max_timestamp = Max('fivemindata__timestamp'))

            if stations.count() == 1:
                station = stations[0]

        # Create the context dict here, so that if the station doesn't 
        # exist, we don't have to do checking at the end of this 
        # method.
        context = {}
        context['station_id'] = station_id
        context['station'] = station
        context['output'] = GRAPHS_OUTPUT
        context['no_timeseries_table_header'] = True

        if station is not None:

            setattr(station, 'elevation_feet', station.elevation * 3.281)

            delta = datetime.timedelta(seconds = 3600 * 6)

            if station.max_timestamp >= (now() - delta):

                st = station.max_timestamp - datetime.timedelta(days = 1)
                st += datetime.timedelta(seconds = 300)
                et = station.max_timestamp
                df = self.fivemin_timeseries(station.station_id, st, et, 
                    station.time_zone, imperial = True)

                timeseries = self._df_as_dict(df, station.time_zone, 
                    missing_value = 0.0)
                current = timeseries[-1]
                current['precip_total'] = df['precip_total'].sum()

                context['timeseries'] = timeseries
                context['current'] = current
                context['since_midnight'] = self.__since_midnight(station, df)

            else:
                setattr(station, 'inactive', True)

        tp = 'nmcc_ziamet/station/index.html'
        t = loader.get_template(tp)
        c = RequestContext(request, context)

        return HttpResponse(t.render(c))


class StationTimeseries(Station):

    def get_station(self, station_id):

        station = None

        stations = Stn.objects.filter(station_id = station_id)\
            .annotate(max_timestamp = Max('hourlydata__timestamp'))\
            .order_by('sdt')
        count = stations.count()

        if count >= 1:
            station = stations[count - 1]

        return station

    def get(self, request, *args, **kwargs):

        station_id = kwargs.get('station_id', None)
        station = self.get_station(station_id)

        context = {}
        context['station_id'] = station_id
        context['station'] = station

        if station is not None:

            form = StationTimeseriesF(kwargs)

            if form.is_valid():

                context.update(form.cleaned_data)

                if context['frequency_id'] == FIVEMIN_FREQUENCY_ID:
                    tsm = self.fivemin_timeseries_as_dict

                elif context['frequency_id'] == HOURLY_FREQUENCY_ID:
                    tsm = self.hourly_timeseries_as_dict

                else:
                    tsm = self.daily_timeseries_as_dict

                if context['output'] == GRAPHS_OUTPUT:
                    missing_value = 0.0

                elif context['output'] == CSV_OUTPUT:
                    missing_value = float('nan')

                else:
                    missing_value = ''

                context['timeseries'] = tsm(station_id, 
                    context['sdt'], 
                    context['edt'], 
                    context['time_zone'], 
                    context['imperial'], 
                    missing_value)
                context['timeseries_num_rows'] = len(context['timeseries'])

                if context['output'] == GRAPHS_OUTPUT:

                    # Do not render the HTML table with a CSS class 
                    # and a header.
                    #context['embedded_timeseries_table'] = False
                    #context['show_timeseries_table_header'] = False

                    template = 'nmcc_ziamet/station/timeseries.html'
                    mimetype = 'text/html'

                elif context['output'] == CSV_OUTPUT:

                    template = 'nmcc_ziamet/station/hd_timeseries.csv'
                    mimetype = 'text/csv'

                    if context['frequency_id'] == FIVEMIN_FREQUENCY_ID:
                        template = 'nmcc_ziamet/station/fivemin_timeseries.csv'

                else:

                    # Render the HTML table with a CSS class and a 
                    # header.
                    context['embedded_timeseries_table'] = True
                    context['show_timeseries_table_header'] = True

                    template = 'nmcc_ziamet/station/timeseries.html'
                    mimetype = 'text/html'

            else:

                context['form'] = form
                template = 'nmcc_ziamet/station/timeseries.html'
                mimetype = 'text/html'

        t = loader.get_template(template)
        c = RequestContext(request, context)

        return HttpResponse(t.render(c), content_type = mimetype)


class StationTimeseriesForToday(StationTimeseries):

    def get(self, request, *args, **kwargs):

        station_id = kwargs.get('station_id', None)
        station = self.get_station(station_id)

        if station is not None:

            ltz = pytz.timezone(station.time_zone)
            today = now().astimezone(ltz)
            today = today.date()

            #kwargs['station_id'] = station_id
            kwargs['frequency_id'] = HOURLY_FREQUENCY_ID
            kwargs['start_date'] = str(today)
            kwargs['end_date'] = str(today)
            kwargs['time_zone'] = station.time_zone
            kwargs['units'] = IMPERIAL_UNITS
            kwargs['output'] = TABLE_OUTPUT

        r = super(StationTimeseriesForToday, self)\
            .get(request, *args, **kwargs)

        return r


class StationTimeseriesPrevious(StationTimeseries):

    def get(self, request, *args, **kwargs):

        form = StationTimeseriesPreviousF(kwargs)

        if form.is_valid():

            kwargs.update(form.cleaned_data)

            r = super(StationTimeseriesPrevious, self)\
                .get(request, *args, **kwargs)

        else:
            r = HttpResponseBadRequest('Bad Request!')

        return r


class StationRequestTimeseries(StationTimeseries):

    def get(self, request, *args, **kwargs):

        station_id = kwargs.get('station_id', None)
        station = self.get_station(station_id)

        context = {}
        context['station_id'] = station_id
        context['station'] = station

        if station is not None:

            tzinfo = pytz.timezone(station.time_zone)
            st = (station.max_timestamp - datetime.timedelta(days = 31))\
                .astimezone(tzinfo)
            et = station.max_timestamp.astimezone(tzinfo)

            sd = st.date()
            ed = et.date()
            initial = {'start_date': sd, 
                'end_date': ed, 
                'time_zone': station.time_zone}

            context['form'] = StationTimeseriesF(initial = initial)

        t = loader.get_template('nmcc_ziamet/station/timeseries.html')
        c = RequestContext(request, context)

        return HttpResponse(t.render(c))

    def post(self, request, *args, **kwargs):

        kwargs['station_id'] = request.POST.get('station_id')
        kwargs['frequency_id'] = request.POST.get('frequency_id')
        kwargs['start_date'] = request.POST.get('start_date')
        kwargs['end_date'] = request.POST.get('end_date')
        kwargs['time_zone'] = request.POST.get('time_zone')
        kwargs['units'] = request.POST.get('units')
        kwargs['output'] = request.POST.get('output')

        r = super(StationRequestTimeseries, self).get(request, *args, **kwargs)

        return r

class StationRGTimeseries(StationTimeseries):

    def get(self, request, *args, **kwargs):

        # The frequency for a Reference ET and GDD request will ALWAYS 
        # be 'daily'.
        kwargs['frequency_id'] = DAILY_FREQUENCY_ID

        station_id = kwargs.get('station_id', None)
        station = self.get_station(station_id)

        context = {}
        context['station_id'] = station_id
        context['station'] = station

        if station is not None:

            form = StationRGTimeseriesF(kwargs)

            if form.is_valid():

                context.update(form.cleaned_data)

                if context['output'] == GRAPHS_OUTPUT:
                    missing_value = 0.0

                elif context['output'] == CSV_OUTPUT:
                    missing_value = float('nan')

                else:
                    missing_value = ''

                context['timeseries'] = self.daily_retgdd_timeseries_as_dict(
                    station_id, 
                    context['sdt'], 
                    context['edt'], 
                    context['time_zone'], 
                    context['imperial'],  
                    missing_value,
                    gdd_tbase = context['gdd_tbase'], 
                    gdd_tmaxcutoff = context['gdd_tmaxcutoff'], 
                    gdd_tmincutoff = context['gdd_tmincutoff'], 
                    cgdd_offset = context['cgdd_offset'])
                context['timeseries_num_rows'] = len(context['timeseries'])

                if context['output'] == GRAPHS_OUTPUT:

                    template = 'nmcc_ziamet/station/retgdd_timeseries.html'
                    mimetype = 'text/html'

                elif context['output'] == CSV_OUTPUT:

                    template = 'nmcc_ziamet/station/retgdd_timeseries.csv'
                    mimetype = 'text/csv'

                else:

                    # Render the HTML table with a CSS class and a 
                    # header.
                    context['embedded_timeseries_table'] = True
                    context['show_timeseries_table_header'] = True

                    template = 'nmcc_ziamet/station/retgdd_timeseries.html'
                    mimetype = 'text/html'

            else:

                context['form'] = form
                template = 'nmcc_ziamet/station/retgdd_timeseries.html'
                mimetype = 'text/html'

        t = loader.get_template(template)
        c = RequestContext(request, context)

        return HttpResponse(t.render(c), content_type = mimetype)


class StationRGTimeseriesPrevious(StationRGTimeseries):

    def get(self, request, *args, **kwargs):

        # The frequency for a Reference ET and GDD request will ALWAYS 
        # be 'daily'.
        kwargs['frequency_id'] = DAILY_FREQUENCY_ID

        form = StationRGTimeseriesPreviousF(kwargs)

        if form.is_valid():

            kwargs.update(form.cleaned_data)

            r = super(StationRGTimeseriesPrevious, self)\
                .get(request, *args, **kwargs)

        else:
            r = HttpResponseBadRequest('Bad Request!')

        return r


class StationRequestRGTimeseries(StationRGTimeseries):

    def get(self, request, *args, **kwargs):

        station_id = kwargs.get('station_id', None)
        station = self.get_station(station_id)

        context = {}
        context['station_id'] = station_id
        context['station'] = station

        if station is not None:

            tzinfo = pytz.timezone(station.time_zone)
            st = (station.max_timestamp - datetime.timedelta(days = 31))\
                .astimezone(tzinfo)
            et = station.max_timestamp.astimezone(tzinfo)

            sd = st.date()
            ed = et.date()
            initial = {'start_date': sd, 
                'end_date': ed, 
                'time_zone': station.time_zone}

            context['form'] = StationRGTimeseriesF(initial = initial)

        t = loader.get_template('nmcc_ziamet/station/retgdd_timeseries.html')
        c = RequestContext(request, context)

        return HttpResponse(t.render(c))

    def post(self, request, *args, **kwargs):

        kwargs['station_id'] = request.POST.get('station_id')
        kwargs['frequency_id'] = request.POST.get('frequency_id')
        kwargs['start_date'] = request.POST.get('start_date')
        kwargs['end_date'] = request.POST.get('end_date')
        kwargs['time_zone'] = request.POST.get('time_zone')
        kwargs['units'] = request.POST.get('units')
        kwargs['output'] = request.POST.get('output')
        kwargs['gdd_tbase'] = request.POST.get('gdd_tbase')
        kwargs['gdd_tmaxcutoff'] = request.POST.get('gdd_tmaxcutoff')
        kwargs['gdd_tmincutoff'] = request.POST.get('gdd_tmincutoff')
        kwargs['cgdd_offset'] = request.POST.get('cgdd_offset')

        r = super(StationRequestRGTimeseries, self)\
            .get(request, *args, **kwargs)

        return r


